import 'dart:convert';
import 'dart:io';

import 'package:archive/archive.dart';
import 'package:archive/archive_io.dart';
import 'package:cli_util/cli_logging.dart';
import 'package:http/http.dart' as http;
import 'package:path/path.dart' as path;

Future<void> download(String package_name, Logger logger,
    String dct_storage_path, String dart2native_path, String pub_path) async {
  var progress_handle = logger.progress('Downloading $package_name from Pub');

  var pub_response = json.decode(
      await http.read('https://pub.dartlang.org/api/packages/$package_name'));

  logger.trace('Package information downloaded');

  var archive_raw = await http.readBytes(pub_response['latest']['archive_url']);

  logger.trace('Archive downloaded');

  var archive_unzipped = GZipDecoder().decodeBytes(archive_raw);
  var archive_untarred = TarDecoder().decodeBytes(archive_unzipped);

  logger.trace('Archive unpacked');

  var work_area =
      path.normalize(dct_storage_path + '/work_area/' + '/$package_name/') +
          '/';

  logger.trace('Work area is $work_area');

  for (final file in archive_untarred) {
    final filename = file.name;
    logger.trace('Writing $filename to disk');
    if (file.isFile) {
      final data = file.content as List<int>;
      File(work_area + filename)
        ..createSync(recursive: true)
        ..writeAsBytesSync(data);
    } else {
      await Directory(work_area + filename).create(recursive: true);
    }
  }

  logger.stdout('Package code downloaded');
  logger.stdout(
      'Building $package_name version ${pub_response['latest']['version']}');

  logger.trace('Downloading dependencies');
  Process.runSync(pub_path, ['get'], workingDirectory: work_area);
  logger.trace('Compiling');
  Process.runSync(dart2native_path, [
    path.normalize(work_area + '/bin/main.dart'),
    '-o',
    path.normalize(work_area + '/bin/$package_name.exe')
  ]);

  logger.stdout(
      'Built $package_name version ${pub_response['latest']['version']}');
  logger.stdout('Cleaning up');

  logger.trace('Copying executable');
  // Directory(path.normalize(
  //         '$dct_storage_path/packages/$package_name/${pub_response['latest']['version']}/'))
  //     .createSync(recursive: true);
  Directory(path.normalize('$dct_storage_path/packages/'))
      .createSync(recursive: true);
  File(path.normalize(work_area + '/bin/$package_name.exe'))
      .renameSync(path.normalize(dct_storage_path +
          // '/packages/$package_name/${pub_response['latest']['version']}/$package_name.exe'));
          '/packages/$package_name.exe'));

  logger.trace('Deleting work area');
  Directory(work_area).deleteSync(recursive: true);

  progress_handle.finish(message: 'Download finished', showTiming: true);
  return;
}
